@@ -5,8 +5,6 @@
 package org.chromium.weblayer;
 
 import android.os.RemoteException;
-import android.view.SurfaceControlViewHost;
-import android.view.View;
 
 import androidx.annotation.NonNull;
 import androidx.annotation.Nullable;
@@ -14,9 +12,10 @@ import androidx.annotation.Nullable;
 import org.chromium.weblayer_private.interfaces.APICallException;
 import org.chromium.weblayer_private.interfaces.IBrowser;
 import org.chromium.weblayer_private.interfaces.IBrowserClient;
+import org.chromium.weblayer_private.interfaces.IBrowserFragment;
+import org.chromium.weblayer_private.interfaces.IMediaRouteDialogFragment;
 import org.chromium.weblayer_private.interfaces.IRemoteFragment;
 import org.chromium.weblayer_private.interfaces.ITab;
-import org.chromium.weblayer_private.interfaces.ObjectWrapper;
 import org.chromium.weblayer_private.interfaces.StrictModeWorkaround;
 
 import java.util.Set;
@@ -47,10 +46,8 @@ class Browser {
     // Set to null once destroyed (or for tests).
     private IBrowser mImpl;
     private final ObserverList<TabListCallback> mTabListCallbacks;
-    private final UrlBarController mUrlBarController;
 
-    private final ObserverList<BrowserControlsOffsetCallback> mBrowserControlsOffsetCallbacks;
-    private final ObserverList<BrowserRestoreCallback> mBrowserRestoreCallbacks;
+    private final ObserverList<TabInitializationCallback> mTabInitializationCallbacks;
 
     private static int sMaxNavigationsPerTabForInstanceState;
 
@@ -81,24 +78,19 @@ class Browser {
     protected Browser() {
         mImpl = null;
         mTabListCallbacks = null;
-        mUrlBarController = null;
-        mBrowserControlsOffsetCallbacks = null;
-        mBrowserRestoreCallbacks = null;
+        mTabInitializationCallbacks = null;
     }
 
     // Constructor for browserfragment to inject the {@code tabListCallback} on startup.
-    Browser(IBrowser impl, @Nullable TabListCallback tabListCallback) {
+    Browser(IBrowser impl) {
         mImpl = impl;
         mTabListCallbacks = new ObserverList<TabListCallback>();
-        if (tabListCallback != null) {
-            mTabListCallbacks.addObserver(tabListCallback);
-        }
-        mBrowserControlsOffsetCallbacks = new ObserverList<BrowserControlsOffsetCallback>();
-        mBrowserRestoreCallbacks = new ObserverList<BrowserRestoreCallback>();
+        mTabInitializationCallbacks = new ObserverList<TabInitializationCallback>();
+    }
 
+    void initializeState() {
         try {
             mImpl.setClient(new BrowserClientImpl());
-            mUrlBarController = new UrlBarController(mImpl.getUrlBarController());
         } catch (RemoteException e) {
             throw new APICallException(e);
         }
@@ -114,6 +106,28 @@ class Browser {
         return mImpl;
     }
 
+    /**
+     * Returns remote counterpart for the BrowserFragment: an {@link IBrowserFragment}.
+     */
+    IBrowserFragment connectFragment() {
+        try {
+            return mImpl.getBrowserFragmentImpl();
+        } catch (RemoteException e) {
+            throw new APICallException(e);
+        }
+    }
+
+    /**
+     * Returns the remote counterpart of MediaRouteDialogFragment.
+     */
+    /* package */ IMediaRouteDialogFragment createMediaRouteDialogFragment() {
+        try {
+            return mImpl.createMediaRouteDialogFragmentImpl();
+        } catch (RemoteException e) {
+            throw new APICallException(e);
+        }
+    }
+
     /**
      * Returns true if this Browser has been destroyed.
      */
@@ -321,133 +335,25 @@ class Browser {
     }
 
     /**
-     * Adds a BrowserRestoreCallback.
+     * Adds a TabInitializationCallback.
      *
-     * @param callback The BrowserRestoreCallback.
+     * @param callback The TabInitializationCallback.
      */
-    public void registerBrowserRestoreCallback(@NonNull BrowserRestoreCallback callback) {
+    public void registerTabInitializationCallback(@NonNull TabInitializationCallback callback) {
         ThreadCheck.ensureOnUiThread();
         throwIfDestroyed();
-        mBrowserRestoreCallbacks.addObserver(callback);
+        mTabInitializationCallbacks.addObserver(callback);
     }
 
     /**
-     * Removes a BrowserRestoreCallback.
+     * Removes a TabInitializationCallback.
      *
-     * @param callback The BrowserRestoreCallback.
+     * @param callback The TabInitializationCallback.
      */
-    public void unregisterBrowserRestoreCallback(@NonNull BrowserRestoreCallback callback) {
+    public void unregisterTabInitializationCallback(@NonNull TabInitializationCallback callback) {
         ThreadCheck.ensureOnUiThread();
         throwIfDestroyed();
-        mBrowserRestoreCallbacks.removeObserver(callback);
-    }
-
-    /**
-     * Sets the View shown at the top of the browser. A value of null removes the view. The
-     * top-view is typically used to show the uri. The top-view scrolls with the page.
-     *
-     * @param view The new top-view.
-     */
-    public void setTopView(@Nullable View view) {
-        ThreadCheck.ensureOnUiThread();
-        throwIfDestroyed();
-        try {
-            mImpl.setTopView(ObjectWrapper.wrap(view));
-        } catch (RemoteException e) {
-            throw new APICallException(e);
-        }
-    }
-
-    /**
-     * Sets the View shown at the top of the browser. The top-view is typically used to show the
-     * uri. This method also allows you to control the scrolling behavior of the top-view by setting
-     * a minimum height it will scroll to, and pinning the top-view to the top of the web contents.
-     *
-     * @param view The new top-view, or null to remove the view.
-     * @param minHeight The minimum height in pixels that the top controls can scoll up to. A value
-     *        of 0 means the top-view should scroll entirely off screen.
-     * @param onlyExpandControlsAtPageTop Whether the top-view should only be expanded when the web
-     *        content is scrolled to the top. A true value makes the top-view behave as though it
-     *        were inserted into the top of the page content. If true, the top-view should NOT be
-     *        used to display the URL, as this will prevent it from expanding in security-sensitive
-     *        contexts where the URL should be visible to the user.
-     * @param animate Whether or not any height/visibility changes that result from this call
-     *        should be animated.
-     */
-    public void setTopView(@Nullable View view, int minHeight, boolean onlyExpandControlsAtPageTop,
-            boolean animate) {
-        ThreadCheck.ensureOnUiThread();
-        throwIfDestroyed();
-        try {
-            mImpl.setTopViewAndScrollingBehavior(
-                    ObjectWrapper.wrap(view), minHeight, onlyExpandControlsAtPageTop, animate);
-        } catch (RemoteException e) {
-            throw new APICallException(e);
-        }
-    }
-
-    /**
-     * Sets the View shown at the bottom of the browser. A value of null removes the view.
-     *
-     * @param view The new bottom-view.
-     */
-    public void setBottomView(@Nullable View view) {
-        ThreadCheck.ensureOnUiThread();
-        throwIfDestroyed();
-        try {
-            mImpl.setBottomView(ObjectWrapper.wrap(view));
-        } catch (RemoteException e) {
-            throw new APICallException(e);
-        }
-    }
-
-    /**
-     * Registers {@link callback} to be notified when the offset of the top or bottom view changes.
-     *
-     * @param callback The BrowserControlsOffsetCallback to notify
-     *
-     * @since 88
-     */
-    public void registerBrowserControlsOffsetCallback(
-            @NonNull BrowserControlsOffsetCallback callback) {
-        ThreadCheck.ensureOnUiThread();
-        throwIfDestroyed();
-        if (WebLayer.getSupportedMajorVersionInternal() < 88) {
-            throw new UnsupportedOperationException();
-        }
-        if (mBrowserControlsOffsetCallbacks.isEmpty()) {
-            try {
-                mImpl.setBrowserControlsOffsetsEnabled(true);
-            } catch (RemoteException e) {
-                throw new APICallException(e);
-            }
-        }
-        mBrowserControlsOffsetCallbacks.addObserver(callback);
-    }
-
-    /**
-     * Removes a BrowserControlsOffsetCallback that was added using {@link
-     * registerBrowserControlsOffsetCallback}.
-     *
-     * @param callback The BrowserControlsOffsetCallback to remove.
-     *
-     * @since 88
-     */
-    public void unregisterBrowserControlsOffsetCallback(
-            @NonNull BrowserControlsOffsetCallback callback) {
-        ThreadCheck.ensureOnUiThread();
-        throwIfDestroyed();
-        if (WebLayer.getSupportedMajorVersionInternal() < 88) {
-            throw new UnsupportedOperationException();
-        }
-        mBrowserControlsOffsetCallbacks.removeObserver(callback);
-        if (mBrowserControlsOffsetCallbacks.isEmpty()) {
-            try {
-                mImpl.setBrowserControlsOffsetsEnabled(false);
-            } catch (RemoteException e) {
-                throw new APICallException(e);
-            }
-        }
+        mTabInitializationCallbacks.removeObserver(callback);
     }
 
     /**
@@ -467,33 +373,6 @@ class Browser {
         }
     }
 
-    /**
-     * Set the minimum surface size of this Browser instance.
-     * Setting this avoids expensive surface resize for a fragment view resize that is within the
-     * minimum size. The trade off is the additional memory and power needed for the larger
-     * surface. For example, for a browser use case, it's likely worthwhile to set the minimum
-     * surface size to the screen size to avoid surface resize when entering and exiting fullscreen.
-     * It is safe to call this before Views are initialized.
-     * Note Android does have a max size limit on Surfaces which applies here as well; this
-     * generally should not be larger than the device screen size.
-     * Note the surface size is increased to the layout size only if both the width and height are
-     * no larger than the minimum surface size. No adjustment is made if the surface size is larger
-     * than the minimum size in one dimension and smaller in the other dimension.
-     * @since 89
-     */
-    public void setMinimumSurfaceSize(int width, int height) {
-        ThreadCheck.ensureOnUiThread();
-        if (WebLayer.getSupportedMajorVersionInternal() < 89) {
-            throw new UnsupportedOperationException();
-        }
-        throwIfDestroyed();
-        try {
-            mImpl.setMinimumSurfaceSize(width, height);
-        } catch (RemoteException e) {
-            throw new APICallException(e);
-        }
-    }
-
     /**
      * Controls how sites are themed when WebLayer is in dark mode. WebLayer considers itself to be
      * in dark mode if the UI_MODE_NIGHT_YES flag of its Resources' Configuration's uiMode field is
@@ -534,63 +413,9 @@ class Browser {
         }
     }
 
-    /**
-     * Returns the UrlBarController.
-     */
-    @NonNull
-    public UrlBarController getUrlBarController() {
-        ThreadCheck.ensureOnUiThread();
-        throwIfDestroyed();
-        return mUrlBarController;
-    }
-
-    /**
-     * Normally when the Browser is detached the visibility of the page is set to hidden. When the
-     * visibility is hidden video may stop, or other side effects may result. At certain times,
-     * such as fullscreen or rotation, it may be necessary to transiently detach the Browser.
-     * Calling this method with a value of false results in WebLayer not hiding the page on the next
-     * detach. Once the Browser is reattached, the value is implicitly reset to true. Calling this
-     * method when the Browser is already detached does nothing.
-     *
-     * @param changeVisibility Whether WebLayer should change visibility as the result of a detach.
-     *
-     * @since 91
-     */
-    public void setChangeVisibilityOnNextDetach(boolean changeVisibility) {
-        ThreadCheck.ensureOnUiThread();
-        if (WebLayer.getSupportedMajorVersionInternal() < 91) {
-            throw new UnsupportedOperationException();
-        }
-        throwIfDestroyed();
+    public void shutdown() {
         try {
-            mImpl.setChangeVisibilityOnNextDetach(changeVisibility);
-        } catch (RemoteException e) {
-            throw new APICallException(e);
-        }
-    }
-
-    /**
-     * Attaches the top-level view to the SurfaceControlViewHost.
-     * @param host The SurfaceControlViewHost created from the host app's SurfaceView.
-     *
-     * @since 105
-     */
-    void setSurfaceControlViewHost(SurfaceControlViewHost host) {
-        ThreadCheck.ensureOnUiThread();
-
-        if (WebLayer.getSupportedMajorVersionInternal() < 105) {
-            throw new UnsupportedOperationException();
-        }
-        try {
-            mImpl.setSurfaceControlViewHost(ObjectWrapper.wrap(host));
-        } catch (RemoteException e) {
-            throw new APICallException(e);
-        }
-    }
-
-    View getContentViewRenderView() {
-        try {
-            return ObjectWrapper.unwrap(mImpl.getContentViewRenderView(), View.class);
+            mImpl.shutdown();
         } catch (RemoteException e) {
             throw new APICallException(e);
         }
@@ -648,20 +473,9 @@ class Browser {
         }
 
         @Override
-        public void onBrowserControlsOffsetsChanged(boolean isTop, int offset) {
-            for (BrowserControlsOffsetCallback callback : mBrowserControlsOffsetCallbacks) {
-                if (isTop) {
-                    callback.onTopViewOffsetChanged(offset);
-                } else {
-                    callback.onBottomViewOffsetChanged(offset);
-                }
-            }
-        }
-
-        @Override
-        public void onRestoreCompleted() {
-            for (BrowserRestoreCallback callback : mBrowserRestoreCallbacks) {
-                callback.onRestoreCompleted();
+        public void onTabInitializationCompleted() {
+            for (TabInitializationCallback callback : mTabInitializationCallbacks) {
+                callback.onTabInitializationCompleted();
             }
         }
     }
